@@ -44,12 +44,25 @@ impl RemoteCommit for GiteaCommit {
 }
 
 impl RemoteEntry for GiteaCommit {
-	fn url(_id: i64, api_url: &str, remote: &Remote, page: i32) -> String {
-		format!(
+	fn url(
+		_id: i64,
+		api_url: &str,
+		remote: &Remote,
+		ref_name: Option<&str>,
+		page: i32,
+	) -> String {
+		let mut url = format!(
 			"{}/api/v1/repos/{}/{}/commits?limit={MAX_PAGE_SIZE}&page={page}",
 			api_url, remote.owner, remote.repo
-		)
+		);
+
+		if let Some(ref_name) = ref_name {
+			url.push_str(&format!("&sha={}", ref_name));
+		}
+
+		url
 	}
+
 	fn buffer_size() -> usize {
 		10
 	}
@@ -106,7 +119,13 @@ impl RemotePullRequest for GiteaPullRequest {
 }
 
 impl RemoteEntry for GiteaPullRequest {
-	fn url(_id: i64, api_url: &str, remote: &Remote, page: i32) -> String {
+	fn url(
+		_id: i64,
+		api_url: &str,
+		remote: &Remote,
+		_ref_name: Option<&str>,
+		page: i32,
+	) -> String {
 		format!(
 			"{}/api/v1/repos/{}/{}/pulls?limit={MAX_PAGE_SIZE}&page={page}&\
 			 state=closed",
@@ -158,9 +177,12 @@ impl RemoteClient for GiteaClient {
 
 impl GiteaClient {
 	/// Fetches the Gitea API and returns the commits.
-	pub async fn get_commits(&self) -> Result<Vec<Box<dyn RemoteCommit>>> {
+	pub async fn get_commits(
+		&self,
+		ref_name: Option<&str>,
+	) -> Result<Vec<Box<dyn RemoteCommit>>> {
 		Ok(self
-			.fetch::<GiteaCommit>(0)
+			.fetch::<GiteaCommit>(0, ref_name)
 			.await?
 			.into_iter()
 			.map(|v| Box::new(v) as Box<dyn RemoteCommit>)
@@ -170,9 +192,10 @@ impl GiteaClient {
 	/// Fetches the Gitea API and returns the pull requests.
 	pub async fn get_pull_requests(
 		&self,
+		ref_name: Option<&str>,
 	) -> Result<Vec<Box<dyn RemotePullRequest>>> {
 		Ok(self
-			.fetch::<GiteaPullRequest>(0)
+			.fetch::<GiteaPullRequest>(0, ref_name)
 			.await?
 			.into_iter()
 			.map(|v| Box::new(v) as Box<dyn RemotePullRequest>)
